# Sherlock version 1.0.0
# QuadraticAssignment.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of Sherlock, a primitive solver for numerical contraint satisfaction problems.
# Sherlock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Sherlock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Sherlock.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Script
# 3) Main

# 1) Imported modules
# -------------------

import string
import sys
import re

# 2) Script
# ---------

numberOfCities = 5

distances = [
  [0, 50, 60, 130, 100],
  [50, 0, 70, 150, 120],
  [60, 70, 0, 80, 40],
  [130, 150, 80, 0, 50],
  [100, 120, 40, 50, 0],
  ]

travels = [
  [0, 10, 15, 12, 8],
  [9, 0, 18, 16, 10],
  [20, 8, 0, 10, 12],
  [10, 15, 11, 0, 22],
  [7, 12, 9, 11, 0],
  ]

def QuadraticAssignment_PrintHeader(output):
  output.write("""/*
 * Quadratic assignment problem
 */

domain City [1, %d]
""" % numberOfCities)

def QuadraticAssignment_PrintProblem(output):
  output.write("problem QuadraticAssignment\n")

  output.write("  City ")  
  for i in range(1, numberOfCities+1):
    if i>1:
      output.write(", ")
    output.write("W%d" % i);
  output.write(";\n");

  for i in range(1, numberOfCities):
    for j in range(i+1, numberOfCities+1):
      costs = []
      for k in range(1, numberOfCities+1):
        for l in range(1, numberOfCities+1):
          if k!=l:
           c = distances[k-1][l-1]*travels[i-1][j-1] + distances[l-1][k-1]*travels[j-1][i-1]
           if not c in costs:
             costs.append(c)
      costs.sort()
      output.write("  {")
      first = True
      for c in costs:
        if first:
          first = False
        else:
          output.write(", ")
        output.write(str(c))
      output.write("} C%d%d;\n" % (i, j))

  output.write("  constraint\n");

  output.write("    alldifferent ")  
  for i in range(1, numberOfCities+1):
    if i>1:
      output.write(", ")
    output.write("W%d" % i);
  output.write(";\n");

  for i in range(1, numberOfCities):
    for j in range(i+1, numberOfCities+1):
      for k in range(1, numberOfCities+1):
        for l in range(1, numberOfCities+1):
          if k!=l:
            c = distances[k-1][l-1]*travels[i-1][j-1] + distances[l-1][k-1]*travels[j-1][i-1]
            output.write("    or W%d!=%d, W%d!=%d, C%d%d==%d;\n" % (i, k, j, l, i, j, c));

  output.write("  minimize ");
  first = True
  for i in range(1, numberOfCities):
    for j in range(i+1, numberOfCities+1):
      if first:
        first = False
      else:
        output.write(" + ")
      output.write("C%d%d" % (i, j))
  output.write(";\n");

  output.write("end\n");

# 3) Main
# -------

fileName = "QuadraticAssignment.csp"
try:
  file = open(fileName, "w")
except:
  sys.stderr.write('Unable to open file "%s"\n' % fileName)
  sys.stderr.flush()
  sys.exit(1)

QuadraticAssignment_PrintHeader(file);
file.write("\n")
QuadraticAssignment_PrintProblem(file)
file.write("\n")
file.flush()
file.close()


